﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinTree;

namespace WindowsFormsApplication13
{
    public enum SelectionModes
    {
        Single,
        Multiple,
    }

    public enum TreeViewTypeFilter
    {
        None,
        Category,
        Activity,
        Filter,
        Organisation,
        Restriction,
        Workshift,
        Code,
        Comment,
        Availability,
        Employee
    }

    public partial class PHTreeView : UserControl
    {
        public PHTreeView()
        {
            InitializeComponent();
            OnSelectionModeChanged();
        }

        #region Private Members

        SelectionModes _selectionMode = SelectionModes.Single;

        List<UltraTreeNode> _selectedNodes_Prev = new List<UltraTreeNode>();

        bool _ItemJustSelected = false;

        #endregion

        #region Properties

        [Description("Set the selection mode: Single, Multiple")]
        [DefaultValue(SelectionModes.Single)]
        public SelectionModes SelectionMode
        {
            get
            {
                return _selectionMode;
            }
            set
            {
                _selectionMode = value; OnSelectionModeChanged();
            }
        }

        public UltraTree TreeView
        {
            get
            {
                return this.ultraTree;
            }
        }

        private Color pTreeColor = Color.White;
        [Browsable(false)]
        public Color TreeColor
        {
            get { return pTreeColor; }
            set
            {
                pTreeColor = value;
                ultraTree.Appearance.BackColor = pTreeColor;
            }
        }

        private bool _allowingMultiSimilarItem = false;
        [DefaultValue(false)]
        public bool SelectSimilarItem
        {
            get
            {
                return _allowingMultiSimilarItem;
            }
            set
            {
                _allowingMultiSimilarItem = value;
            }
        }

        private bool _forFilterSort = false;
        [DefaultValue(false)]
        public bool ForFilterSort
        {
            get
            {
                return _forFilterSort;
            }
            set
            {
                _forFilterSort = value;
            }
        }

        #endregion

        #region UI Events

        private void ultraTree_BeforeExpand(object sender, CancelableNodeEventArgs e)
        {
            if (PHBeforeExpand != null)
            {
                PHBeforeExpand(this, e);
            }
        }

        private void ultraTree_AfterExpand(object sender, NodeEventArgs e)
        {
            if (PHAfterExpand != null)
            {
                PHAfterExpand(this, e);
            }
        }

        private void ultraTree_MouseClick(object sender, MouseEventArgs e)
        {
            if (SelectionMode == SelectionModes.Single && (ModifierKeys & Keys.Control) != 0 && _ItemJustSelected == false)
            {
                var controlElement = (sender as UltraTree).UIElement;

                var nodeTextUIElement = controlElement.ElementFromPoint(e.Location) as NodeTextUIElement;
                if (nodeTextUIElement != null)
                {
                    var item = nodeTextUIElement.GetContext(typeof(UltraTreeNode)) as UltraTreeNode;
                    item.Selected = !item.Selected;
                }
            }
            _ItemJustSelected = false;
        }

        private void ultraTree_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (PHMouseDoubleClick != null)
            {
                PHMouseDoubleClick(sender, e);
            }
        }

        private void ultraTree_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.F4)
            {
                bool oneNodeExpanded = false;
                foreach (UltraTreeNode node in this.ultraTree.Nodes)
                {
                    if (node.Expanded == true)
                    {
                        oneNodeExpanded = true;
                        break;
                    }
                }

                if (oneNodeExpanded == true)
                {
                    this.ultraTree.CollapseAll();
                }
                else
                {
                    this.ultraTree.ExpandAll();
                }
            }
        }

        private void ultraTree_BeforeSelect(object sender, BeforeSelectEventArgs e)
        {
            if (PHBeforeSelect != null)
            {
                PHBeforeSelect(this, e);
            }
        }

        private void ultraTree_AfterSelect(object sender, SelectEventArgs e)
        {
            if (ForFilterSort)
            {
                ultraTree.AfterSelect -= ultraTree_AfterSelect;

                if ((ModifierKeys & Keys.Control) == Keys.None) // This is always a new selection and we do not need to 
                                                                // take into account the previously selected nodes
                {
                    e.NewSelections.OfType<UltraTreeNode>().ForEach(ns => SelectAllChildNode_ForFilterSort(ns));

                    if (SelectSimilarItem) // For UT5.2.12 and SE5.2.50
                    {
                        ultraTree.SelectedNodes.OfType<UltraTreeNode>().Where(n => n.HasNodes == false).ForEach(n => SelectSimilarNode(n));
                    }
                }
                else
                {
                    var newlySelectedNodes = e.NewSelections.OfType<UltraTreeNode>()
                        .Except(_selectedNodes_Prev.OfType<UltraTreeNode>()).ToList();

                    var removedSelectedNodes = _selectedNodes_Prev.OfType<UltraTreeNode>()
                        .Except(e.NewSelections.OfType<UltraTreeNode>()).ToList();

                    newlySelectedNodes.ForEach(nsn => SelectAllChildNode(nsn));

                    if (SelectSimilarItem) // For UT5.2.12 and SE5.2.50
                    {
                        removedSelectedNodes.Where(nsn => nsn.HasNodes == false).ForEach(rsn => UnselectSimilarItems(rsn));
                        ultraTree.SelectedNodes.OfType<UltraTreeNode>().Where(n => n.HasNodes == false).ForEach(n => SelectSimilarNode(n));
                    }
                }

                UnselectParentNodeIfAllChildNotSelected(ultraTree.Nodes);

                _selectedNodes_Prev = ultraTree.SelectedNodes.OfType<UltraTreeNode>().ToList();

                ultraTree.AfterSelect += ultraTree_AfterSelect;
            }
            else
            {
                if (_selectionMode == SelectionModes.Multiple)
                {
                    if (this.ultraTree.ActiveNode.Selected == true)
                    {
                        SelectAllChildNode(this.ultraTree.ActiveNode);
                    }
                    else
                    {
                        UnselectAllParentNode(this.ultraTree.ActiveNode);
                    }

                    SelectParentIfAllChildSelected(this.ultraTree.ActiveNode);
                }
            }

            if (PHAfterSelect != null)
            {
                PHAfterSelect(sender, e);
            }

            _ItemJustSelected = true;
        }

        private void ultraTree_Leave(object sender, EventArgs e)
        {
            if (PHLeave != null)
            {
                PHLeave(sender, e);
            }
        }

        private void ultraTree_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (PHKeyPress != null)
            {
                PHKeyPress(this, e);
            }
        }

        private void ultraTree_PreviewKeyDown(object sender, PreviewKeyDownEventArgs e)
        {
            if (PHPreviewKeyDown != null)
            {
                PHPreviewKeyDown(this, e);
            }
        }

        #endregion

        #region Private Methods

        private void OnSelectionModeChanged()
        {
            switch (_selectionMode)
            {
                case SelectionModes.Single:
                    this.ultraTree.Override.SelectionType = SelectType.SingleAutoDrag;
                    this.ultraTree.SelectionBehavior = SelectionBehavior.UseOverride;
                    break;
                case SelectionModes.Multiple:
                    this.ultraTree.SelectionBehavior = SelectionBehavior.ExtendedAcrossCollections;
                    break;
                default:
                    break;
            }
        }

        private void ExpandNodeAndParent(UltraTreeNode node)
        {
            node.Expanded = true;

            if (node.Parent != null)
            {
                ExpandNodeAndParent(node.Parent);
            }
        }

        private void SelectAllChildNode(UltraTreeNode node)
        {
            if (node.HasNodes)
            {
                foreach (UltraTreeNode childNode in node.Nodes)
                {
                    SelectAllChildNode(childNode);
                }
            }
            this.ultraTree.ActiveNode = node;
            node.Selected = true;
        }

        private void UnselectAllParentNode(UltraTreeNode node)
        {
            if (node.Parent != null)
            {
                UnselectAllParentNode(node.Parent);
            }
            this.ultraTree.ActiveNode = node;
            node.Selected = false;
        }

        private void SelectParentIfAllChildSelected(UltraTreeNode node)
        {
            UltraTreeNode parentNode = node.Parent;

            if (parentNode == null) return;

            bool isAllChildSelected = true;

            foreach (UltraTreeNode childNode in parentNode.Nodes)
            {
                isAllChildSelected &= childNode.Selected;
            }

            if (isAllChildSelected == true)
            {
                parentNode.Selected = true;
            }
        }

        private void SelectAllChildNode_ForFilterSort(UltraTreeNode node)
        {
            node.Selected = true;
            ExpandNodeAndParent(node);

            foreach (UltraTreeNode childNode in node.Nodes)
            {
                SelectAllChildNode_ForFilterSort(childNode);
            }
        }

        private void UnselectParentNodeIfAllChildNotSelected(TreeNodesCollection nodes)
        {
            foreach (var node in nodes)
            {
                UnselectParentNodeIfAllChildNotSelected(node.Nodes);

                if (node.Nodes.OfType<UltraTreeNode>().All(n => n.Selected) == false)
                {
                    node.Selected = false;
                }
            }
        }

        private void SelectSimilarNode(UltraTreeNode node)
        {
            var nodeToBeSelected = FindSimilarNodes(ultraTree.Nodes, node);
            nodeToBeSelected.ForEach(ntbs =>
                {
                    ntbs.Selected = true;
                    ExpandNodeAndParent(ntbs);
                });
        }

        private void UnselectSimilarItems(UltraTreeNode node)
        {
            var nodeToBeUnselected = FindSimilarNodes(ultraTree.Nodes, node);
            nodeToBeUnselected.ForEach(ntbs => ntbs.Selected = false);
        }

        private List<UltraTreeNode> FindSimilarNodes(TreeNodesCollection nodes, UltraTreeNode nodeToCompare)
        {
            var result = new List<UltraTreeNode>();

            foreach (var node in nodes)
            {
                if (node.Tag == nodeToCompare.Tag)
                {
                    result.Add(node);
                }

                result.AddRange(FindSimilarNodes(node.Nodes, nodeToCompare));
            }

            return result;
        }

        #endregion

        #region External Events

        [Category("PrimHeure")]
        public event KeyPressEventHandler PHKeyPress;
        [Category("PrimHeure")]
        public event BeforeNodeSelectEventHandler PHBeforeSelect;
        [Category("PrimHeure")]
        public event AfterNodeSelectEventHandler PHAfterSelect;
        [Category("PrimHeure")]
        public event EventHandler PHLeave;
        [Category("PrimHeure")]
        public event MouseEventHandler PHMouseDoubleClick;
        [Category("PrimHeure")]
        public event EventHandler<NodeEventArgs> PHAfterExpand;
        [Category("PrimHeure")]
        public event EventHandler<CancelableNodeEventArgs> PHBeforeExpand;
        [Category("PrimHeure")]
        public event PreviewKeyDownEventHandler PHPreviewKeyDown;

        #endregion

    }
}
